"""
 Coloring Book nitty gritty..

 The main internals of a colornig book. Each book consists of a color
 selection box and a canvas. In addition are buttons for paging through
 the images in the book, a button for taking a snapshot of the canvas, and
 a box that shows the currently selected color.

 Each canvas is a page from the coloring book composed of image layers. Each
 layer is an area of the coloring book that can be colored.
"""

import glob, random, sys, tempfile, text, ui
from locals import *

class book:
	def __init__ ( self, color = None ):
		self.changed = True
		self.pages = None
		self.messages = pygame.sprite.RenderClear ()
		self.groups = [ pygame.sprite.RenderClear (), pygame.sprite.RenderClear (), pygame.sprite.RenderClear () ]
		if not color:
			color = DEFAULT_COLOR
		self.Selected_color = selected_color ( self.groups[1], [ 124, 434 ], color )
		self.Left_button = left_button ( self.groups[1], [ 5, 230 ] )
		self.Right_button = right_button ( self.groups[1], [ 624, 230 ] )
		self.Undo_button = undo_button ( self.groups[1], [ 500, 436 ] )
		self.Snapshot_button = snapshot_button ( self.groups[1], [ 550, 438 ] )
		self.Colors = colors ( self.groups[1], [ 169, 434 ] )
		self.Cursor = cursor ( self.groups[2], pygame.mouse.get_pos () )
		self.Chrome = chrome ( self.groups[0] )
		self.start_canvas ()

	def list_pages ( self ):
		if not self.pages:
			pages = glob.glob ( 'pages/*.py' )
			if pages:
				self.pages = pages
			else:
				sys.exit ()
		return self.pages

	def start_page ( self ):
		self.page = 0
		return self.list_pages ()[0]

	def prev_page ( self ):
		if self.page == 0:
			self.page = len ( self.list_pages () ) - 1
		else:
			self.page = self.page - 1
		return self.list_pages ()[self.page]

	def next_page ( self ):	
		if self.page == len ( self.list_pages () ) - 1:
			self.page = 0
		else:
			self.page = self.page + 1
		return self.list_pages ()[self.page]

	def start_canvas ( self ):
		self.Canvas = canvas ( self.groups[1], [ 19, 19 ], self.start_page () )
		self.init_history ()
		self.Canvas.draw ()

	def prev_canvas ( self ):
		self.Canvas.kill ()
		self.Canvas = canvas ( self.groups[1], [ 19, 19 ], self.prev_page () )
		self.init_history ()
		self.Canvas.draw ()

	def next_canvas ( self ):
		self.Canvas.kill ()
		self.Canvas = canvas ( self.groups[1], [ 19, 19 ], self.next_page () )
		self.init_history ()
		self.Canvas.draw ()

	def move_cursor ( self, pos, buttons ):
		self.changed = True
		self.Cursor.set_pos ( pos )
		for group in self.groups:
			for sprite in group.sprites ():
				sprite.check_cursor ( pos, buttons )

	def check_click ( self, pos ):
		for group in self.groups:
			for sprite in group.sprites ():
				if sprite.check_click ( pos, self ):
					self.changed = True
					return
	def init_history ( self ):
		self.history = []
		for layer in self.Canvas.layers:
			self.history.append ( [ layer, layer.color ] )

	def draw ( self ):
		for sprite in self.messages.sprites ():
			self.changed = True
			sprite.shift ()
		if self.changed:
			self.changed = False
			return self.groups
		return None

	def paint_layer ( self, layer ):
		self.history.append ( [ layer, layer.color ] )
		layer.set_color ( self.Selected_color.color )
		self.Canvas.draw ()

	def set_color ( self, color ):
		if color is CANVAS_MASK:
			color[2] = color[2] - 1
		self.Selected_color.set_color ( color )

	def undo ( self ):
		try:
			last = self.history.pop ()
			last[0].set_color ( last[1] )
			self.Canvas.draw ()
		except IndexError:
			pass

	def snapshot ( self ):
		canvas = self.Canvas.draw ()
		fd, filename = tempfile.mkstemp ( '.tga', dir = SNAPSHOT_DIR )
		pygame.image.save ( canvas, filename )
		self.messages.add ( message ( self.groups[2], 'Snapshot saved as %s.' % ( filename ), self ) )

class layer:
	def __init__ ( self, surface, rect, offset, color = None, name = None ):
		self.surface = surface
		self.rect = rect
		self.offset = offset
		self.size = ( rect[2], rect[3] )
		self.background = pygame.Surface ( self.size ).convert ()
		if not color:
			color = BACKGROUND_COLOR
		self.set_color ( color )
		if not name:
			name = ''
		self.name = name

	def check_click ( self, pos ):
		if pos[0] >= self.offset[0] and pos[1] >= self.offset[1] and pos[0] <= ( self.offset[0] + self.size[0] ) and pos[1] <= ( self.offset[1] + self.size[1] ):
			p = ( pos[0] - self.offset[0] + self.rect[0], pos[1] - self.offset[1] + self.rect[1] )
			try:
#				if self.surface.get_at ( p ) == LAYER_MASK:
				if self.surface.get_at ( p ) != CANVAS_MASK:
					return True
			except IndexError:
				pass
		return False

	def set_color ( self, color ):
		self.color = color
		self.background.fill ( color )
		self.surface.set_colorkey ( LAYER_MASK )
		self.background.blit ( self.surface, ( 0, 0 ), self.rect )

	def draw ( self, surface ):
		self.background.set_colorkey ( CANVAS_MASK )
		surface.blit ( self.background, self.offset )

class canvas ( pygame.sprite.Sprite ):
	def __init__ ( self, group, pos, page ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.image = pygame.Surface ( CANVAS_SIZE ).convert ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.page = page
		self.background ()
		self.load_layers ()

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def draw ( self ):
		for layer in self.layers:
			layer.draw ( self.image )
		self.image = ui.lower_surface ( self.image )
		return self.image

	def background ( self ):
		background = pygame.Surface ( DISPLAY_SIZE ).convert ()
		background.fill ( LAYER_MASK )
		self.layers = [ layer ( background, ( 0, 0, 600, 400 ), ( 0, 0 ), name = 'background' ) ]

	def load_layers ( self ):
		page = {}
		execfile ( self.page, page )
		img = pygame.image.load ( page['image'] ).convert ()
		for m in page['map']:
			self.layers.append ( layer ( img, m[0], m[1] ) )

	def check_cursor ( self, pos, buttons ):
		return False

	def check_click ( self, pos, cb ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			pos = [ pos[0] - self.rect.left, pos[1] - self.rect.top ]
#			layers = self.layers[:]
#			layers.reverse ()
#			for layer in layers:
			for layer in self.layers[::-1]:
				if layer.check_click ( pos ):
					cb.paint_layer ( layer )
					return True
		return False

class colors ( pygame.sprite.Sprite ):
	def __init__ ( self, group, pos ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.image = pygame.image.load ( COLORS_IMAGE ).convert ()
		self.image.set_colorkey ( CANVAS_MASK )
		self.image = ui.lower_surface ( self.image )
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def check_cursor ( self, pos, buttons ):
		return False

	def check_click ( self, pos, cb ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			try:
				pos = [ pos[0] - self.rect.left, pos[1] - self.rect.top ]
				color = self.image.get_at ( pos )
				cb.set_color ( color )
				return True
			except IndexError:
				return False
		return False

class cursor ( pygame.sprite.Sprite ):
	def __init__ ( self, group, pos ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.image = pygame.image.load ( CURSOR_IMAGE ).convert ()
		self.image.set_colorkey ( CANVAS_MASK )
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def check_cursor ( self, pos, buttons ):
		return False

	def check_click ( self, pos, cb ):
		return False

class undo_button ( pygame.sprite.Sprite ):
	def __init__ ( self, group, pos ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.highlighted = False
		self.image1 = pygame.image.load ( UNDO_IMAGE ).convert ()
		self.image1.set_colorkey ( CANVAS_MASK )
		self.image2 = pygame.image.load ( UNDO_HIGHLIGHT_IMAGE ).convert ()
		self.image2.set_colorkey ( CANVAS_MASK )
		self.image = self.image1
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def check_cursor ( self, pos, buttons ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			self.highlighted = True
			self.image = self.image2
			return True
		if self.highlighted:
			self.image = self.image1
			self.highlighted = False
		return False

	def check_click ( self, pos, cb ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			cb.undo ()
			return True
		return False

class snapshot_button ( pygame.sprite.Sprite ):
	def __init__ ( self, group, pos ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.highlighted = False
		self.image1 = pygame.image.load ( SNAPSHOT_IMAGE ).convert ()
		self.image1.set_colorkey ( CANVAS_MASK )
		self.image2 = pygame.image.load ( SNAPSHOT_HIGHLIGHT_IMAGE ).convert ()
		self.image2.set_colorkey ( CANVAS_MASK )
		self.image = self.image1
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def check_cursor ( self, pos, buttons ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			self.highlighted = True
			self.image = self.image2
			return True
		if self.highlighted:
			self.image = self.image1
			self.highlighted = False
		return False

	def check_click ( self, pos, cb ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			cb.snapshot ()
			return True
		return False

class left_button ( pygame.sprite.Sprite ):
	def __init__ ( self, group, pos ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.highlighted = False
		buttons = pygame.image.load ( ARROWS_IMAGE ).convert ()
		hbuttons = pygame.image.load ( ARROWS_HIGHLIGHT_IMAGE ).convert ()
		self.image1 = pygame.Surface ( [ 10, 18 ] )
		self.image1.blit ( buttons, ( 0, 0 ), ( 0, 0, 10, 18 ) )
		self.image1.set_colorkey ( CANVAS_MASK )
		self.image2 = pygame.Surface ( [ 10, 18 ] )
		self.image2.blit ( hbuttons, ( 0, 0 ), ( 0, 0, 10, 18 ) )
		self.image2.set_colorkey ( CANVAS_MASK )
		self.image = self.image1
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def check_cursor ( self, pos, buttons ):
		if self.rect.left <= pos[0] and self.rect.right >= pos[0]:
			self.highlighted = True
			self.image = self.image2
			return True
		if self.highlighted:
			self.image = self.image1
			self.highlighted = False
		return False

	def check_click ( self, pos, cb ):
		if self.rect.left <= pos[0] and self.rect.right >= pos[0]:
			cb.prev_canvas ()
			return True
		return False

class right_button ( pygame.sprite.Sprite ):
	def __init__ ( self, group, pos ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.highlighted = False
		buttons = pygame.image.load ( ARROWS_IMAGE ).convert ()
		hbuttons = pygame.image.load ( ARROWS_HIGHLIGHT_IMAGE ).convert ()
		self.image1 = pygame.Surface ( [ 10, 18 ] )
		self.image1.blit ( buttons, ( 0, 0 ), ( 13, 0, 10, 18 ) )
		self.image1.set_colorkey ( CANVAS_MASK )
		self.image2 = pygame.Surface ( [ 10, 18 ] )
		self.image2.blit ( hbuttons, ( 0, 0 ), ( 13, 0, 10, 18 ) )
		self.image2.set_colorkey ( CANVAS_MASK )
		self.image = self.image1
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def check_cursor ( self, pos, buttons ):
		if self.rect.left <= pos[0] and self.rect.right >= pos[0]:
			self.highlighted = True
			self.image = self.image2
			return True
		if self.highlighted:
			self.image = self.image1
			self.highlighted = False
		return False

	def check_click ( self, pos, cb ):
		if self.rect.left <= pos[0] and self.rect.right >= pos[0]:
			cb.next_canvas ()
			return True
		return False

class selected_color ( pygame.sprite.Sprite ):
	def __init__ ( self, group, pos, color ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.image = pygame.Surface ( ( 30, 30 ) ).convert ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.set_color ( color )

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def set_color ( self, color ):
		self.color = color
		self.image.fill ( self.color )
		self.image = ui.lower_surface ( self.image )

	def check_cursor ( self, pos, buttons ):
		return False

	def check_click ( self, pos, cb ):
		return False

class chrome ( pygame.sprite.Sprite ):
	def __init__ ( self, group ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.image = pygame.Surface ( DISPLAY_SIZE ).convert ()
		self.image.fill ( CHROME_COLOR )
		self.image = ui.raise_surface ( self.image )
		self.rect = self.image.get_rect ()

	def check_cursor ( self, pos, buttons ):
		return False

	def check_click ( self, pos, cb ):
		return False

class message ( pygame.sprite.Sprite ):
	def __init__ ( self, group, message, cb ):
		pygame.sprite.Sprite.__init__ ( self, group )
		self.image = text.textOutline ( DEFAULT_FONT, message, FONT_COLOR, OUTLINE_COLOR )
		self.rect = self.image.get_rect ()
		self.cb = cb
		self.alpha = 255
		self.start_pos ()

	def set_pos ( self, pos ):
		self.rect.topleft = pos

	def start_pos ( self ):
		w = ( self.cb.Canvas.rect.width - self.rect.width ) / 2 + 20
		h = ( self.cb.Canvas.rect.height - self.rect.height ) / 2 + 20
		self.set_pos ( [ w, h ] )

	def shift ( self ):
		self.alpha = self.alpha - 5
		if self.alpha < 1:
			self.kill ()
		else:
			self.image.set_alpha ( self.alpha )

	def check_cursor ( self, pos, buttons ):
		return False

	def check_click ( self, pos, cb ):
		return False
